Package: LaplacesDemon
Version: 16.1.8
Title: Complete Environment for Bayesian Inference
Authors@R: c(person("Byron", "Hall", role = "aut"),
    person("Martina", "Hall", role = "aut"),
    person(family="Statisticat, LLC", role = c("aut", "cph")),
    person(given = "Charles J.", family="Geyer", role = c("ctb", "cph"),
    comment="for TR method in LaplaceApproximation, derived from trust::trust"),
    person(given="Eric", family="Brown", role = "ctb"),
    person(given="Richard", family="Hermanson", role = "ctb"),
    person(given="Emmanuel", family="Charpentier", role = "ctb"),
    person(given="Daniel", family="Heck", role = "ctb"),
    person(given="Stephane", family="Laurent", role = "ctb"),
    person(given="Quentin F.", family="Gronau", role = "ctb"),
    person(given="Henrik", family="Singmann", 
    email="singmann+LaplacesDemon@gmail.com", role="cre"))
Depends: R (>= 3.0.0)
Imports: parallel, grDevices, graphics, stats, utils
Suggests: KernSmooth
ByteCompile: TRUE
Description: Provides a complete environment for Bayesian inference using a variety of different samplers (see ?LaplacesDemon for an overview).
License: MIT + file LICENSE
URL: https://github.com/LaplacesDemonR/LaplacesDemon
BugReports: https://github.com/LaplacesDemonR/LaplacesDemon/issues
NeedsCompilation: no
Packaged: 2026-02-17 12:00:50 UTC; singmann
Author: Byron Hall [aut],
  Martina Hall [aut],
  Statisticat, LLC [aut, cph],
  Charles J. Geyer [ctb, cph] (for TR method in LaplaceApproximation,
    derived from trust::trust),
  Eric Brown [ctb],
  Richard Hermanson [ctb],
  Emmanuel Charpentier [ctb],
  Daniel Heck [ctb],
  Stephane Laurent [ctb],
  Quentin F. Gronau [ctb],
  Henrik Singmann [cre]
Maintainer: Henrik Singmann <singmann+LaplacesDemon@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-17 15:20:07 UTC
