import(parallel)
export(ABB,
       AcceptanceRate,
       as.covar,
       as.indicator.matrix,
       as.initial.values,
       as.inverse,
       as.parm.matrix,
       as.parm.names,
       as.positive.definite,
       as.positive.semidefinite,
       as.ppc,
       as.symmetric.matrix,
       BayesFactor,
       BayesianBootstrap,
       BayesTheorem,
       BigData,
       Blocks,
       BMK.Diagnostic,
       burnin,
       caterpillar.plot,
       CenterScale,
       cloglog,
       Combine,
       cond.plot,
       Consort,
       Cov2Cor,
       Cov2Prec,
       CovEstim,
       CSF,
       dalaplace,
       dallaplace,
       daml,
       dbern,
       dcat,
       dcrmrf,
       ddirichlet,
       deburn,
       delicit,
       de.Finetti.Game,
       dgpd,
       dgpois,
       dhalfcauchy,
       dhalfnorm,
       dhalft,
       dhs,
       dhuangwand,
       dhuangwandc,
       dhyperg,
       dinvbeta,
       dinvchisq,
       dinvgamma,
       dinvgaussian,
       dinvmatrixgamma,
       dinvwishart,
       dinvwishartc,
       dlaplace,
       dlaplacem,
       dlaplacep,
       dlasso,
       dllaplace,
       dlnormp,
       dmatrixgamma,
       dmatrixnorm,
       dmvc,
       dmvcc,
       dmvcp,
       dmvcpc,
       dmvl,
       dmvlc,
       dmvn,
       dmvnc,
       dmvnp,
       dmvnpc,
       dmvpe,
       dmvpec,
       dmvpolya,
       dmvt,
       dmvtc,
       dmvtp,
       dmvtpc,
       dnorminvwishart,
       dnormlaplace,
       dnormm,
       dnormp,
       dnormv,
       dnormwishart,
       dpareto,
       dpe,
       dsdlaplace,
       dsiw,
       dslaplace,
       dStick,
       dst,
       dstp,
       dtrunc,
       dwishart,
       dwishartc,
       dyangberger,
       dyangbergerc,
       dzellner,
       ESS,
       elicit,
       extrunc,
       GaussHermiteCubeRule,
       GaussHermiteQuadRule,
       Gelfand.Diagnostic,
       Gelman.Diagnostic,
       Geweke.Diagnostic,
       GIV,
       Hangartner.Diagnostic,
       Heidelberger.Diagnostic,
       Hermite,
       Hessian,
       IAT,
       Importance,
       interval,
       invcloglog,
       invlogit,
       invloglog,
       is.amodal,
       is.appeased,
       is.bayesfactor,
       is.bayesian,
       is.bimodal,
       is.blocks,
       is.bmk,
       is.constant,
       is.constrained,
       is.data,
       is.demonoid,
       is.demonoid.hpc,
       is.demonoid.ppc,
       is.demonoid.val,
       is.hangartner,
       is.heidelberger,
       is.importance,
       is.iterquad,
       is.iterquad.ppc,
       is.juxtapose,
       is.laplace,
       is.laplace.ppc,
       is.miss,
       is.model,
       is.multimodal,
       is.pmc,
       is.pmc.ppc,
       is.pmc.val,
       is.positive.definite,
       is.positive.semidefinite,
       is.posteriorchecks,
       is.proper,
       is.raftery,
       is.rejection,
       is.sensitivity,
       is.square.matrix,
       is.stationary,
       is.symmetric.matrix,
       is.trimodal,
       is.unimodal,
       is.vb,
       is.vb.ppc,
       IterativeQuadrature,
       Jacobian,
       joint.density.plot,
       joint.pr.plot,
       Juxtapose,
       KLD,
       KS.Diagnostic,
       LaplaceApproximation,
       LaplacesDemon,
       LaplacesDemon.hpc,
       LaplacesDemon.RAM,
       Levene.Test,
       LML,
       logadd,
       logdet,
       logit,
       loglog,
       LossMatrix,
       lower.triangle,
       LPL.interval,
       MCSE,
       MCSS,
       MinnesotaPrior,
       MISS,
       Mode,
       Model.Spec.Time,
       Modes,
       PMC,
       PMC.RAM,
       PosteriorChecks,
       Prec2Cov,
       palaplace,
       pallaplace,
       partial,
       ppareto,
       pbern,
       phalfcauchy,
       phalfnorm,
       phalft,
       plaplace,
       plaplacem,
       plaplacep,
       pllaplace,
       plotMatrix,
       plotSamples,
       plnormp,
       pnormm,
       pnormp,
       pnormv,
       ppe,
       prec2sd,
       prec2var,
       psdlaplace,
       pslaplace,
       pst,
       pstp,
       ptrunc,
       p.interval,
       qalaplace,
       qallaplace,
       qbern,
       qcat,
       qhalfcauchy,
       qhalfnorm,
       qhalft,
       qlaplace,
       qlaplacep,
       qllaplace,
       qlnormp,
       qnormp,
       qnormv,
       qpareto,
       qpe,
       qsdlaplace,
       qslaplace,
       qst,
       qstp,
       qtrunc,
       Raftery.Diagnostic,
       ralaplace,
       rallaplace,
       raml,
       rbern,
       rcat,
       rcrmrf,
       rdirichlet,
       read.matrix,
       RejectionSampling,
       rgpd,
       rhalfcauchy,
       rhalfnorm,
       rhalft,
       rhs,
       rhuangwand,
       rhuangwandc,
       rinvbeta,
       rinvchisq,
       rinvgamma,
       rinvgaussian,
       rinvwishart,
       rinvwishartc,
       rlaplace,
       rlaplacem,
       rlaplacep,
       rlasso,
       rllaplace,
       rlnormp,
       rmatrixnorm,
       rmvc,
       rmvcc,
       rmvcp,
       rmvcpc,
       rmvl,
       rmvlc,
       rmvn,
       rmvnc,
       rmvnp,
       rmvnpc,
       rmvpe,
       rmvpec,
       rmvpolya,
       rmvt,
       rmvtc,
       rmvtp,
       rmvtpc,
       rnorminvwishart,
       rnormlaplace,
       rnormm,
       rnormp,
       rnormv,
       rnormwishart,
       rpareto,
       rpe,
       rsdlaplace,
       rsiw,
       rslaplace,
       rst,
       rStick,
       rstp,
       rtrunc,
       rwishart,
       rwishartc,
       rzellner,
       sd2prec,
       sd2var,
       SensitivityAnalysis,
       server_Listening,
       SIR,
       SparseGrid,
       Stick,
       Thin,
       TransitionMatrix,
       tr,
       upper.triangle,
       Validate,
       VariationalBayes,
       vartrunc,
       var2prec,
       var2sd,
       WAIC,
       .colVars,
       .iqagh,
       .iqaghsg,
       .iqcagh,
       .laaga,
       .labfgs,
       .labhhh,
       .lacg,
       .ladfp,
       .lahar,
       .lahj,
       .lalbfgs,
       .lalm,
       .lanm,
       .lanr,
       .lapso,
       .larprop,
       .lasgd,
       .lasoma,
       .laspg,
       .lasr1,
       .latr,
       .mcmcadmg,
       .mcmcafss,
       .mcmcagg,
       .mcmcahmc,
       .mcmcaies,
       .mcmcam,
       .mcmcamm,
       .mcmcamm.b,
       .mcmcamwg,
       .mcmccharm,
       .mcmcdemc,
       .mcmcdram,
       .mcmcdrm,
       .mcmcess,
       .mcmcgibbs,
       .mcmcgg,
       .mcmcggcp,
       .mcmcggcpp,
       .mcmcggdp,
       .mcmcggdpp,
       .mcmcharm,
       .mcmchmc,
       .mcmchmcda,
       .mcmcim,
       .mcmcinca,
       .mcmcmala,
       .mcmcmcmcmc,
       .mcmcmtm,
       .mcmcmwg,
       .mcmcnuts,
       .mcmcohss,
       .mcmcram,
       .mcmcrdmh,
       .mcmcrefractive,
       .mcmcrj,
       .mcmcrss,
       .mcmcrwm,
       .mcmcsamwg,
       .mcmcsgld,
       .mcmcslice,
       .mcmcsmwg,
       .mcmcthmc,
       .mcmctwalk,
       .mcmcuess,
       .mcmcusamwg,
       .mcmcusmwg,
       .rowVars,
       .vbsalimans2)

S3method("plot","bmk")
S3method("plot","demonoid")
S3method("plot","demonoid.hpc")
S3method("plot","demonoid.ppc")
S3method("plot","importance")
S3method("plot","iterquad")
S3method("plot","iterquad.ppc")
S3method("plot","juxtapose")
S3method("plot","laplace")
S3method("plot","laplace.ppc")
S3method("plot","miss")
S3method("plot","pmc")
S3method("plot","pmc.ppc")
S3method("plot","vb")
S3method("plot","vb.ppc")
S3method("predict","demonoid")
S3method("predict","iterquad")
S3method("predict","laplace")
S3method("predict","pmc")
S3method("predict","vb")
S3method("print","demonoid")
S3method("print","heidelberger")
S3method("print","iterquad")
S3method("print","laplace")
S3method("print","miss")
S3method("print","pmc")
S3method("print","raftery")
S3method("print","vb")
S3method("summary","demonoid.ppc")
S3method("summary","iterquad.ppc")
S3method("summary","laplace.ppc")
S3method("summary","miss")
S3method("summary","pmc.ppc")
S3method("summary","vb.ppc")
importFrom("grDevices", "col2rgb", "colorRampPalette", "dev.off",
             "heat.colors", "palette", "pdf", "rgb")
  importFrom("graphics", "abline", "axis", "barplot", "boxplot",
             "dotchart", "hist", "image", "layout", "legend", "lines",
             "panel.smooth", "par", "plot", "plot.new", "plot.window",
             "points", "polygon", "rect", "segments", "smoothScatter",
             "strwidth", "symbols", "text")
  importFrom("stats", "ARMAacf", "Gamma", "acf", "aggregate", "approx",
             "ar", "ar.yw", "as.dist", "chisq.test", "complete.cases",
             "cor", "cov", "cov.wt", "cutree", "dbeta", "dbinom",
             "density", "dexp", "dgamma", "dist", "dnorm", "ecdf", "fft",
             "glm", "hclust", "integrate", "ks.test", "lm", "lowess",
             "median", "optim", "optimize", "pbinom", "pgamma", "pnorm",
             "predict", "pt", "qbinom", "qf", "qgamma", "qnorm", "qt",
             "quantile", "rbeta", "rbinom", "rchisq", "residuals",
             "rexp", "rgamma", "rmultinom", "rnorm", "rt", "runif", "sd",
             "spline", "start", "ts", "uniroot", "var", "rlnorm")
  importFrom("utils", "capture.output", "count.fields", "head",
             "object.size", "tail")
